/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.compomics.util.experiment.identification;

import com.compomics.util.experiment.biology.AminoAcid;
import com.compomics.util.experiment.biology.Protein;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.JProgressBar;

/**
 * This class contains basic statistics of the amino acid composition of a
 * database. It is its own factory based on the sequenceFactory.
 *
 * @author marc
 */
public class AAStats implements Serializable {

    static final long serialVersionUID = 4600458866001677754L;
    private HashMap<String, Integer> aaOccurences = new HashMap<String, Integer>();
    private int totalAA = 0;
    private HashMap<String, HashMap<String, Integer>> aaAfter = new HashMap<String, HashMap<String, Integer>>();
    private HashMap<String, Integer> totalAAAfter = new HashMap<String, Integer>();
    private HashMap<String, HashMap<String, Integer>> aaBefore = new HashMap<String, HashMap<String, Integer>>();
    private HashMap<String, Integer> totalAABefore = new HashMap<String, Integer>();
    private ArrayList<String> implementedAa = AminoAcid.getAminoAcids();

    public AAStats() {
    }

    public void loadStatsForProtein(String accession) throws IOException {
        ArrayList<String> accessions = new ArrayList<String>();
        accessions.add(accession);
        loadStatsForProteins(accessions, null);
    }

    public void loadStatsForProteins(ArrayList<String> accessions, JProgressBar progressBar) throws IOException {
        SequenceFactory sequenceFactory = SequenceFactory.getInstance();
        if (progressBar != null) {
            progressBar.setIndeterminate(false);
            progressBar.setMaximum(accessions.size());
        }
        Protein protein;
        String aa, aa1;
        int n;
        for (String accession : accessions) {
            if (!SequenceFactory.isDecoy(accession)) {
                protein = sequenceFactory.getProtein(accession);
                String[] aas = protein.getSequence().split("");
                for (int i = 0; i < aas.length; i++) {
                    aa = aas[i];
                    if (implementedAa.contains(aa)) {
                        n = 0;
                        if (aaOccurences.containsKey(aa)) {
                            n = aaOccurences.get(aa);
                        }
                        aaOccurences.put(aa, n + 1);
                        totalAA++;
                        if (i > 0) {
                            aa1 = aas[i - 1];
                            if (implementedAa.contains(aa1)) {
                                if (!aaBefore.containsKey(aa)) {
                                    aaBefore.put(aa, new HashMap<String, Integer>());
                                    totalAABefore.put(aa, 0);
                                }
                                n = 0;
                                if (aaBefore.get(aa).containsKey(aa1)) {
                                    n = aaBefore.get(aa).get(aa1);
                                }
                                aaBefore.get(aa).put(aa1, n + 1);
                                n = totalAABefore.get(aa) + 1;
                                totalAABefore.put(aa, n);
                            }
                        }
                        if (i < aas.length - 1) {
                            aa1 = aas[i + 1];
                            if (implementedAa.contains(aa1)) {
                                if (!aaAfter.containsKey(aa)) {
                                    aaAfter.put(aa, new HashMap<String, Integer>());
                                    totalAAAfter.put(aa, 0);
                                }
                                n = 0;
                                if (aaAfter.get(aa).containsKey(aa1)) {
                                    n = aaAfter.get(aa).get(aa1);
                                }
                                aaAfter.get(aa).put(aa1, n + 1);
                                n = totalAAAfter.get(aa) + 1;
                                totalAAAfter.put(aa, n);
                            }
                        }
                    }
                }
            }
            if (progressBar != null) {
                progressBar.setValue(progressBar.getValue() + 1);
            }
        }
        if (progressBar != null) {
            progressBar.setIndeterminate(true);
        }
    }

    public void loadStatsFromFactory(JProgressBar progressBar) throws IOException {
        SequenceFactory sequenceFactory = SequenceFactory.getInstance();
        ArrayList<String> accessions = sequenceFactory.getAccessions();
        loadStatsForProteins(accessions, progressBar);
    }

    public int getnAA(String aa) {
        if (aaOccurences.containsKey(aa)) {
            return aaOccurences.get(aa);
        } else {
            return 0;
        }
    }

    public int getnAAAfter(String aaRef, String aaAfter) {
        if (this.aaAfter.containsKey(aaRef) && this.aaAfter.get(aaRef).containsKey(aaAfter)) {
            return this.aaAfter.get(aaRef).get(aaAfter);
        }
        return 0;
    }

    public boolean containsAA(String aa) {
        return aaOccurences.containsKey(aa) && aaOccurences.get(aa) != 0;
    }

    public int getnAABefore(String aaRef, String aaBefore) {
        if (this.aaBefore.containsKey(aaRef) && this.aaBefore.get(aaRef).containsKey(aaBefore)) {
            return this.aaBefore.get(aaRef).get(aaBefore);
        }
        return 0;
    }

    public double getpAA(String aa) {
        if (aaOccurences.containsKey(aa)) {
            return ((double) aaOccurences.get(aa)) / totalAA;
        } else {
            return 0;
        }
    }

    public double getpAAAfter(String aaRef, String aaAfter) {
        if (this.aaAfter.containsKey(aaRef) && this.aaAfter.get(aaRef).containsKey(aaAfter)) {
            return ((double) this.aaAfter.get(aaRef).get(aaAfter)) / totalAAAfter.get(aaRef);
        }
        return 0;
    }

    public double getpAABefore(String aaRef, String aaBefore) {
        if (this.aaBefore.containsKey(aaRef) && this.aaBefore.get(aaRef).containsKey(aaBefore)) {
            return ((double) this.aaBefore.get(aaRef).get(aaBefore)) / totalAABefore.get(aaRef);
        }
        return 0;
    }
}
